from direct.showbase.ShowBase import ShowBase

from panda3d.core import NodePath
from panda3d.bullet import BulletWorld, BulletPlaneShape, BulletRigidBodyNode, BulletSoftBodyNode

class Game(ShowBase):
    def __init__(self):
        ShowBase.__init__(self)

        self.world = BulletWorld()
        self.world.set_gravity((0, 0, -9.81))

        bullet_node_ground = BulletRigidBodyNode('ground')
        bullet_node_ground.add_shape(BulletPlaneShape((0, 0, 1), 0))
        self.world.attach_rigid_body(bullet_node_ground)

        model = loader.load_model('models/suzanne.bam')
        geom = model.node().modify_geom(0)

        bullet_soft = BulletSoftBodyNode.make_tri_mesh(self.world.get_world_info(), geom)
        bullet_soft.link_geom(geom)
        bullet_soft.generate_bending_constraints(3)
        bullet_soft.get_cfg().set_positions_solver_iterations(2)
        bullet_soft.randomize_constraints()
        bullet_soft.set_total_mass(50, True)
        self.world.attach_soft_body(bullet_soft)

        model_soft = NodePath(bullet_soft)
        model_soft.set_pos((0, 10, 5))
        model_soft.reparent_to(render)
        model.reparent_to(model_soft)

        taskMgr.add(self.update, 'updateWorld')

    def update(self, task):
        self.world.do_physics(base.clock.get_dt(), 10, 0.008)
        return task.cont

game = Game()
game.run()

